<?php
/*

This is a cookbook receipe for exporting PmWiki files into static HTML.
This is useful for several reasons:
1. Your hosting facility does not allow PHP files
2. For security purpose you don't like exposing the wiki interface on the 
   internet 
while still getting most of the benefits of PmWiki (collaborative editting 
in the intranet, use of skins, etc).

For example exported websites, see:
 - http://www.physics.ox.ac.uk/users/santoso

Author: Yusdi Santoso

Dependencies:
 - Windows (tested with WinXP)
 - wget.exe 
 - For SCP need pscp.exe (tested with 0.59.0.0)
 - For FTP need nftpput.exe (tested with 3.2.0.11)

Installation:
 - Copy export.php in the same folder as pmwiki.php
 - Copy exportConfig.php in the local folder for security purpose
 - Copy the necessary binary files (see dependencies) to your webserver local drive
 - Tweak exportConfig.php to suit your need.

History:
 - 2007/07/18: first create

TODO:
 - Make this cross platform, currently only works in Windows
 - More robust error handling
 - Some kind of progress indication will be good

*/

require_once('local/exportConfig.php');

function _exec($cmd) {
    exec($cmd, $output);
    return $output;
}

function _copydir($srcDir, $destDir) {
    $contents = scandir($srcDir);
    if( !is_dir($destDir) ) {
        mkdir($destDir, 0, TRUE);
    }
    for( $i=0;$i<count($contents);$i++ ) {
        $item = $contents[$i];
        $realSrcPath = "$srcDir/$item";
        $realDestPath = "$destDir/$item";
        if( $item=='.' || $item=='..' ) continue;

        if( is_dir($realSrcPath) ) {
            _copydir($realSrcPath, $realDestPath);
        } else {
            copy($realSrcPath, $realDestPath);
        }
    }
}

function exportSCP($info) {
    if( file_exists($info['bin']) ) {
        $cmd = "{$info['bin']} -r -pw {$info['pwd']} \"{$info['local_folder']}/*\" {$info['user']}@{$info['host']}:\"{$info['remote_folder']}\"";
        _exec($cmd);
    }
}

function exportFTP($info) {
    //Uses NCFTP binary
    if( file_exists($info['bin']) ) {
        $cmd = "{$info['bin']} -R -u {$info['user']} -p {$info['pwd']} {$info['host']} \"{$info['remote_folder']}\" \"{$info['local_folder']}/*\"";
        _exec($cmd); 
    }
}

function exportCopy($info) {
    _copydir($info['local_folder'], $info['remote_folder']);
}

//Make sure this page is never cached by the browser
header("Cache-Control: no-cache, must-revalidate");
header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");

$AVAILABLE_CONNECTION_TYPES = array(
    'SCP'=>exportSCP,
    'FTP'=>exportFTP,
    'COPY'=>exportCopy
);


$COMMON_WGET_OPTS = "-nv -nd -nH -p ".
    "--convert-links ".
    "--html-extension ".
    "--recursive ".
    "--level=$WGET_REC_LEVEL ".
    "--reject \"*edit*,*diff*,*print*,*export*\" ";

// Clean the destination folder
$destFolder = "{$LOCAL_WEBROOT}{$temp}";
_exec("rmdir /s/q \"{$destFolder}\"");
_exec("mkdir \"{$destFolder}\"");

// Download all of the necessary files
foreach($ENTRY_POINTS as $URL=>$DEST ) {
    $cmd = "{$WGET} {$COMMON_WGET_OPTS} ".
        "--directory-prefix=\"{$LOCAL_WEBROOT}{$DEST}\" ".
        "{$URL}";

    _exec($cmd);
}

// Rename files from index.php@n=Main.Publications.1.html to Main_Publications.html
$files = scandir("${LOCAL_WEBROOT}{$temp}");
foreach($files as $file) {
    if( preg_match("/([^=]*)\.html$/", $file, $matches) ) {
        $newFileName = preg_replace("/[0-9]+\./", "", $matches[0]);
        $origFileName = "{$LOCAL_WEBROOT}{$temp}/$file";
        $destFileName = "{$LOCAL_WEBROOT}{$temp}/{$newFileName}";
        $fin = fopen($origFileName, 'r');
        $fout = fopen($destFileName, 'w');
        $content = fread($fin, filesize($origFileName));
        // Simplify file names
        $content = preg_replace("/index\.php@n=([^0-9]*)\.[0-9]+\.html/", '${1}.html', $content);
        // Remove wikicmd
        $content = preg_replace('/id="wikicmds"/', 'id="wikicmds" style="visibility:hidden;"', $content);
        $content = preg_replace('/<a class=\'wikilink\' [^>]*>edit SideBar<\/a>/', '', $content);
        // Rewrite all reference to localhost
        foreach($REWRITE_URLS as $srcURL=>$destURL) {
            $srcPattern = str_replace('/','\/', $srcURL);
            $content = preg_replace('/'.$srcPattern.'/', $destURL, $content);
        }
        fwrite($fout, $content);
        fclose($fin);
        fclose($fout);
        unlink($origFileName);
    }
}

// Rename index.php.html to index.html
rename("{$destFolder}/index.php.html", "{$destFolder}/index.html");

// Copy files to destination directory
$AVAILABLE_CONNECTION_TYPES[$connectionType]($connectionInfo);

?>
<html>
<body>
<p>
Done
</p>
<p>
Visit the new website <a href="<?= $newBaseURL ?>/index.html">here</a>
</p>
</body>
</html>
